new;
library pgraph;

#include corr.src;

N = 5;

let sigma = 0.20 0.20 0.20 0.20 0.20;
let S0 = 100 100 100 100 100;
r = 0.05;
tau = 3/12;

K = 5;
L = 105;

proc Payoff(S);
  local S1,S2,S3,S4,S5,payoff;
  S1 = S[.,1];
  S2 = S[.,2];
  S3 = S[.,3];
  S4 = S[.,4];
  S5 = S[.,5];

  payoff = (S1 - S2 + S3 - S4 - K) .* (S5 .> L);
  payoff = payoff .* (payoff .>= 0);

  retp( PayOff );
endp;

rndseed 123;
Ns = 10000|50;

rho = seqa(-1,0.1,21);
rho[8] = -0.2;
option = miss(zeros(rows(rho),1),0);

i = 1;
do until i > rows(rho);

  C = constantCorrelation(rho[i],N);
  if C == error(0);
    i = i + 1;
    continue;
  endif;

  tmp = zeros(Ns[2],1);

  j = 1;
  do until j > Ns[2];
    S = simulate_mGBM(S0,r,sigma,C,0,tau,Ns[1]);  
    tmp[j] = exp(-r*tau)*meanc(Payoff(S));
    j = j + 1;
  endo;

  option[i] = meanc(tmp);

  i = i + 1;
endo;

max = maxindc(option);
max_option = option[max];
max_rho = constantCorrelation(rho[max],N);

M = 25;
option2 = zeros(M,1);
rho2 = zeros(M,N*(N+1)/2);

i = 1;
do until i > M;
  C = RandomCorrelation2(N,0,-1);
  rho2[i,.] = vech(C)';
  tmp = zeros(Ns[2],1);

  j = 1;
  do until j > Ns[2];
    S = simulate_mGBM(S0,r,sigma,C,0,tau,Ns[1]);  
    tmp[j] = exp(-r*tau)*meanc(Payoff(S));
    j = j + 1;
  endo;

  option2[i] = meanc(tmp);

  i = i + 1;
endo;

max2 = maxindc(option2);
max2_option = option2[max2];
max2_rho = xpnd(rho2[max2,.]');

M = 25;
option3 = zeros(M,1);
rho3 = zeros(M,N*(N+1)/2);

i = 1;
do until i > M;
  C = RandomCorrelation(N,0,0);
  rho3[i,.] = vech(C)';
  tmp = zeros(Ns[2],1);

  j = 1;
  do until j > Ns[2];
    S = simulate_mGBM(S0,r,sigma,C,0,tau,Ns[1]);  
    tmp[j] = exp(-r*tau)*meanc(Payoff(S));
    j = j + 1;
  endo;

  option3[i] = meanc(tmp);

  i = i + 1;
endo;

max3 = maxindc(option3);
max3_option = option3[max3];
max3_rho = xpnd(rho3[max3,.]');

output file = ma.out reset;

print max_rho; print; print ftos(max_option,"                 Option price = %lf",5,3);
print "------------------------------------------------------";
print max2_rho; print; print ftos(max2_option,"                 Option price = %lf",5,3);
print "------------------------------------------------------";
print max3_rho; print; print ftos(max3_option,"                 Option price = %lf",5,3);

output off;

option2 = selif(option2, option2 .> max_option);
e2 = ones(rows(option2),1);

option3 = selif(option3, option3 .> max_option);
e3 = ones(rows(option3),1);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  fonts("simplex simgrma");
  xlabel("\202r\201");
  ylabel("Option price");
  xtics(-1,1,0.25,5);
  ytics(0,4,0.5,5);
  _pline = e2.*(1~6~-1)~option2~e2~option2~e2.*(1~15~0) | 
           e3.*(1~3~-1)~option3~e3~option3~e3.*(1~15~0) ;
  graphprt("-c=1 -cf=ma.ps");
  xy(rho,option);


proc simulate_mGBM(S0,mu,sigma,rho,t0,TT,Ns);
  local ST,N,dt,dw,Pchol;

  S0 = S0'; mu = mu'; sigma = sigma';

  N = maxc(cols(S0)|cols(mu)|cols(sigma));
  dt = TT - t0;

  if rho == 1;
    dw = sqrt(dt) .* rndn(Ns,1) .* ones(1,N);
  elseif rho == -1 and N == 2;
    dw = sqrt(dt) .* (1~-1) .* rndn(Ns,1);
  else; 
    Pchol = chol(rho);
    dw = sqrt(dt) .* rndn(Ns,N)*Pchol;
  endif;

  ST = S0 .* exp( (mu-0.5*sigma^2) .* dt + sigma .* dw );

  retp(ST);
endp;
 
